Module: RESTFramework::BaseModelControllerMixin::ClassMethods

Defined in:
lib/rest_framework/controller_mixins/models.rb

Constant Summary collapse

IGNORE_VALIDATORS_WITH_KEYS =
[:if, :unless].freeze

Instance Method Summary collapse

Instance Method Details

#get_field_config(f) ⇒ Object

Get a field's config, including defaults.



110
111
112
113
114
115
116
117
118
119
120
121
122
123
124
125
126
127
128
129
130
131
132
# File 'lib/rest_framework/controller_mixins/models.rb', line 110

def get_field_config(f)
  config = self.field_config&.dig(f.to_sym) || {}

  # Default sub-fields if field is an association.
  if ref = self.get_model.reflections[f]
    model = ref.klass
    columns = model.columns_hash
    config[:sub_fields] ||= RESTFramework::Utils.sub_fields_for(ref)

    # Serialize very basic metadata about sub-fields.
    config[:sub_fields_metadata] = config[:sub_fields].map { |sf|
      v = {}

      if columns[sf]
        v[:kind] = "column"
      end

      next [sf, v]
    }.to_h.compact.presence
  end

  return config.compact
end

#get_fieldsObject

Get fields without any action context. Always fallback to columns at the class level.



95
96
97
98
99
100
101
102
103
104
105
106
107
# File 'lib/rest_framework/controller_mixins/models.rb', line 95

def get_fields
  if self.fields.is_a?(Hash)
    return RESTFramework::Utils.parse_fields_hash(
      self.fields, self.get_model, exclude_associations: self.exclude_associations
    )
  end

  return self.fields || (
    self.get_model ? RESTFramework::Utils.fields_for(
      self.get_model, exclude_associations: self.exclude_associations
    ) : []
  )
end

#get_fields_metadataObject

Get metadata about the resource's fields.



135
136
137
138
139
140
141
142
143
144
145
146
147
148
149
150
151
152
153
154
155
156
157
158
159
160
161
162
163
164
165
166
167
168
169
170
171
172
173
174
175
176
177
178
179
180
181
182
183
184
185
186
187
188
189
190
191
192
193
194
195
196
197
198
199
200
201
202
203
204
205
206
207
208
209
210
211
212
213
214
215
216
217
218
219
220
221
222
223
224
225
226
227
228
229
230
231
232
233
234
235
236
237
238
239
# File 'lib/rest_framework/controller_mixins/models.rb', line 135

def 
  # Get metadata sources.
  model = self.get_model
  fields = self.get_fields.map(&:to_s)
  columns = model.columns_hash
  column_defaults = model.column_defaults
  reflections = model.reflections
  attributes = model._default_attributes

  return fields.map { |f|
    # Initialize metadata to make the order consistent.
     = {
      type: nil,
      kind: nil,
      label: self.get_label(f),
      primary_key: nil,
      required: nil,
      read_only: nil,
    }

    # Determine `primary_key` based on model.
    if model.primary_key == f
      [:primary_key] = true
    end

    # Determine `type`, `required`, `label`, and `kind` based on schema.
    if column = columns[f]
      [:kind] = "column"
      [:type] = column.type
      [:required] = true unless column.null
    end

    # Determine `default` based on schema; we use `column_defaults` rather than `columns_hash`
    # because these are casted to the proper type.
    column_default = column_defaults[f]
    unless column_default.nil?
      [:default] = column_default
    end

    # Extract details from the model's attributes hash.
    if attributes.key?(f) && attribute = attributes[f]
      unless .key?(:default)
        default = attribute.value_before_type_cast
        [:default] = default unless default.nil?
      end
      [:kind] ||= "attribute"

      # Get any type information from the attribute.
      if type = attribute.type
        [:type] ||= type.type

        # Get enum variants.
        if type.is_a?(ActiveRecord::Enum::EnumType)
          [:enum_variants] = type.send(:mapping)
        end
      end
    end

    # Get association metadata.
    if ref = reflections[f]
      [:kind] = "association"
      begin
        pk = ref.active_record_primary_key
      rescue ActiveRecord::UnknownPrimaryKey
      end
      [:association] = {
        macro: ref.macro,
        class_name: ref.class_name,
        foreign_key: ref.foreign_key,
        primary_key: pk,
        polymorphic: ref.polymorphic?,
        table_name: ref.table_name,
        options: ref.options.presence,
      }.compact
    end

    # Determine if this is just a method.
    if model.method_defined?(f)
      [:kind] ||= "method"
    end

    # Collect validator options into a hash on their type, while also updating `required` based
    # on any presence validators.
    model.validators_on(f).each do |validator|
      kind = validator.kind
      options = validator.options

      # Reject validator if it includes keys like `:if` and `:unless` because those are
      # conditionally applied in a way that is not feasible to communicate via the API.
      next if IGNORE_VALIDATORS_WITH_KEYS.any? { |k| options.key?(k) }

      # Update `required` if we find a presence validator.
      [:required] = true if kind == :presence

      [:validators] ||= {}
      [:validators][kind] ||= []
      [:validators][kind] << options
    end

    # Serialize any field config.
    [:config] = self.get_field_config(f).presence

    next [f, .compact]
  }.to_h
end

#get_label(s) ⇒ Object

Override `get_label` to include ActiveRecord i18n-translated column names.



90
91
92
# File 'lib/rest_framework/controller_mixins/models.rb', line 90

def get_label(s)
  return self.get_model.human_attribute_name(s, default: super)
end

#get_model(from_get_recordset: false) ⇒ Object

Get the model for this controller.



64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
81
82
83
84
85
86
87
# File 'lib/rest_framework/controller_mixins/models.rb', line 64

def get_model(from_get_recordset: false)
  return @model if @model
  return (@model = self.model) if self.model

  # Try to determine model from controller name.
  begin
    return @model = self.name.demodulize.chomp("Controller").singularize.constantize
  rescue NameError
  end

  # Delegate to the recordset's model, if it's defined. This option prevents infinite recursion.
  unless from_get_recordset
    # Instantiate a new controller to get the recordset.
    controller = self.new
    controller.request = ActionController::TestRequest.new
    controller.params = {}

    if (recordset = controller.get_recordset)
      return @model = recordset.klass
    end
  end

  return nil
end

#get_options_metadataObject

Get a hash of metadata to be rendered in the `OPTIONS` response. Cache the result.



242
243
244
245
246
247
248
# File 'lib/rest_framework/controller_mixins/models.rb', line 242

def 
  return super().merge(
    {
      fields: self.,
    },
  )
end

#rrf_finalizeObject

Define any behavior to execute at the end of controller definition. :nocov:



286
287
288
289
290
291
292
293
294
295
296
297
# File 'lib/rest_framework/controller_mixins/models.rb', line 286

def rrf_finalize
  super
  self.setup_delegation
  # self.setup_channel

  if RESTFramework.config.freeze_config
    self::RRF_BASE_MODEL_CONTROLLER_CONFIG.keys.each { |k|
      v = self.send(k)
      v.freeze if v.is_a?(Hash) || v.is_a?(Array)
    }
  end
end

#setup_delegationObject



250
251
252
253
254
255
256
257
258
259
260
261
262
263
264
265
266
267
268
269
270
271
272
273
274
275
276
277
278
279
280
281
282
# File 'lib/rest_framework/controller_mixins/models.rb', line 250

def setup_delegation
  # Delegate extra actions.
  self.extra_actions&.each do |action, config|
    next unless config.is_a?(Hash) && config[:delegate]
    next unless self.get_model.respond_to?(action)

    self.define_method(action) do
      model = self.class.get_model

      if model.method(action).parameters.last&.first == :keyrest
        return api_response(model.send(action, **params))
      else
        return api_response(model.send(action))
      end
    end
  end

  # Delegate extra member actions.
  self.extra_member_actions&.each do |action, config|
    next unless config.is_a?(Hash) && config[:delegate]
    next unless self.get_model.method_defined?(action)

    self.define_method(action) do
      record = self.get_record

      if record.method(action).parameters.last&.first == :keyrest
        return api_response(record.send(action, **params))
      else
        return api_response(record.send(action))
      end
    end
  end
end