Class: RESTFramework::NativeSerializer

Inherits:
BaseSerializer show all
Defined in:
lib/rest_framework/serializers.rb

Overview

This serializer uses `.serializable_hash` to convert objects to Ruby primitives (with the top-level being either an array or a hash).

Direct Known Subclasses

NativeModelSerializer

Instance Attribute Summary

Attributes inherited from BaseSerializer

#object

Class Method Summary collapse

Instance Method Summary collapse

Methods inherited from BaseSerializer

#serializable_hash

Constructor Details

#initialize(object = nil, many: nil, model: nil, **kwargs) ⇒ NativeSerializer

Returns a new instance of NativeSerializer.



32
33
34
35
36
37
38
39
40
41
42
43
44
45
46
47
48
49
# File 'lib/rest_framework/serializers.rb', line 32

def initialize(object=nil, many: nil, model: nil, **kwargs)
  super(object, **kwargs)

  if many.nil?
    # Determine if we are dealing with many objects or just one.
    @many = @object.is_a?(Enumerable)
  else
    @many = many
  end

  # Determine model either explicitly, or by inspecting @object or @controller.
  @model = model
  @model ||= @object.class if @object.is_a?(ActiveRecord::Base)
  @model ||= @object[0].class if
    @many && @object.is_a?(Enumerable) && @object.is_a?(ActiveRecord::Base)

  @model ||= @controller.get_model if @controller
end

Class Method Details

.[](key) ⇒ Object

Allow a serializer class to be used as a hash directly in a nested serializer config.



242
243
244
245
# File 'lib/rest_framework/serializers.rb', line 242

def self.[](key)
  @_nested_config ||= self.new.get_serializer_config
  return @_nested_config[key]
end

.[]=(key, value) ⇒ Object



247
248
249
250
# File 'lib/rest_framework/serializers.rb', line 247

def self.[]=(key, value)
  @_nested_config ||= self.new.get_serializer_config
  return @_nested_config[key] = value
end

.filter_subcfg(subcfg, except: nil, only: nil, additive: false) ⇒ Object

Helper to filter (mutate) a single subconfig for specific keys.



89
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
110
111
112
113
114
115
116
117
118
119
120
121
122
123
124
125
126
127
128
129
130
131
132
133
134
135
136
137
138
139
# File 'lib/rest_framework/serializers.rb', line 89

def self.filter_subcfg(subcfg, except: nil, only: nil, additive: false)
  return subcfg unless except || only
  return subcfg unless subcfg || additive
  raise "Cannot pass `only` and `additive` to filter_subcfg." if only && additive

  if subcfg.is_a?(Array)
    subcfg = subcfg.map(&:to_sym)
    if except
      if additive
        # Only add fields which are not already included.
        subcfg += except - subcfg
      else
        subcfg -= except
      end
    elsif only
      # Ignore `additive` in an `only` context, since it could causing data leaking.
      unless additive
        subcfg.select! { |c| c.in?(only) }
      end
    end
  elsif subcfg.is_a?(Hash)
    # Additive doesn't make sense in a hash context since we wouldn't know the values.
    unless additive
      if except
        subcfg.symbolize_keys!
        subcfg.reject! { |k, _v| k.in?(except) }
      elsif only
        subcfg.symbolize_keys!
        subcfg.select! { |k, _v| k.in?(only) }
      end
    end
  elsif !subcfg
    if additive && except
      subcfg = except
    end
  else  # Subcfg is a single element (assume string/symbol).
    subcfg = subcfg.to_sym

    if except
      if subcfg.in?(except)
        subcfg = [] unless additive
      elsif additive
        subcfg = [subcfg, *except]
      end
    elsif only && !additive && !subcfg.in?(only)  # Protect only/additive data-leaking.
      subcfg = []
    end
  end

  return subcfg
end

Instance Method Details

#[](key) ⇒ Object

Allow a serializer instance to be used as a hash directly in a nested serializer config.



231
232
233
234
# File 'lib/rest_framework/serializers.rb', line 231

def [](key)
  @_nested_config ||= self.get_serializer_config
  return @_nested_config[key]
end

#[]=(key, value) ⇒ Object



236
237
238
239
# File 'lib/rest_framework/serializers.rb', line 236

def []=(key, value)
  @_nested_config ||= self.get_serializer_config
  return @_nested_config[key] = value
end

#_get_raw_serializer_configObject

Get the raw serializer config.



189
190
191
192
193
194
195
196
197
198
199
200
201
202
203
204
205
206
207
208
209
210
211
212
213
214
215
# File 'lib/rest_framework/serializers.rb', line 189

def _get_raw_serializer_config
  # Return a locally defined serializer config if one is defined.
  if local_config = self.get_local_native_serializer_config
    return local_config
  end

  # Return a serializer config if one is defined on the controller.
  if serializer_config = self.get_controller_native_serializer_config
    return serializer_config
  end

  # If the config wasn't determined, build a serializer config from model fields.
  fields = @controller.get_fields if @controller
  if fields
    if @model
      columns, methods = fields.partition { |f| f.in?(@model.column_names) }
    else
      columns = fields
      methods = []
    end

    return {only: columns, methods: methods}
  end

  # By default, pass an empty configuration, allowing the serialization of all columns.
  return {}
end

#filter_except(cfg) ⇒ Object

Helper to filter out configuration properties based on the :except query parameter.



142
143
144
145
146
147
148
149
150
151
152
153
154
155
156
157
158
159
160
161
162
163
164
165
166
167
168
169
170
171
172
173
174
175
176
177
178
179
180
181
182
183
184
185
186
# File 'lib/rest_framework/serializers.rb', line 142

def filter_except(cfg)
  return cfg unless @controller

  except_param = @controller.class.try(:native_serializer_except_query_param)
  only_param = @controller.class.try(:native_serializer_only_query_param)
  if except_param && except = @controller.request.query_parameters[except_param].presence
    except = except.split(",").map(&:strip).map(&:to_sym)

    unless except.empty?
      # Duplicate the cfg to avoid mutating class state.
      cfg = cfg.deep_dup

      # Filter `only`, `except` (additive), `include`, and `methods`.
      if cfg[:only]
        cfg[:only] = self.class.filter_subcfg(cfg[:only], except: except)
      else
        cfg[:except] = self.class.filter_subcfg(cfg[:except], except: except, additive: true)
      end
      cfg[:include] = self.class.filter_subcfg(cfg[:include], except: except)
      cfg[:methods] = self.class.filter_subcfg(cfg[:methods], except: except)
    end
  elsif only_param && only = @controller.request.query_parameters[only_param].presence
    only = only.split(",").map(&:strip).map(&:to_sym)

    unless only.empty?
      # Duplicate the cfg to avoid mutating class state.
      cfg = cfg.deep_dup

      # For the `except` part of the serializer, we need to append any columns not in `only`.
      model = @controller.get_model
      except_cols = model&.column_names&.map(&:to_sym)&.reject { |c| c.in?(only) }

      # Filter `only`, `except` (additive), `include`, and `methods`.
      if cfg[:only]
        cfg[:only] = self.class.filter_subcfg(cfg[:only], only: only)
      else
        cfg[:except] = self.class.filter_subcfg(cfg[:except], except: except_cols, additive: true)
      end
      cfg[:include] = self.class.filter_subcfg(cfg[:include], only: only)
      cfg[:methods] = self.class.filter_subcfg(cfg[:methods], only: only)
    end
  end

  return cfg
end

#get_actionObject

Get controller action, if possible.



52
53
54
# File 'lib/rest_framework/serializers.rb', line 52

def get_action
  return @controller&.action_name&.to_sym
end

#get_controller_native_serializer_configObject

Helper to get a native serializer configuration from the controller.



76
77
78
79
80
81
82
83
84
85
86
# File 'lib/rest_framework/serializers.rb', line 76

def get_controller_native_serializer_config
  return nil unless @controller

  if @many == true
    controller_serializer = @controller.class.try(:native_serializer_plural_config)
  elsif @many == false
    controller_serializer = @controller.class.try(:native_serializer_singular_config)
  end

  return controller_serializer || @controller.class.try(:native_serializer_config)
end

#get_local_native_serializer_configObject

Get a locally defined native serializer configuration, if one is defined.



57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
# File 'lib/rest_framework/serializers.rb', line 57

def get_local_native_serializer_config
  action = self.get_action

  if action && self.action_config
    # Index action should use :list serializer config if :index is not provided.
    action = :list if action == :index && !self.action_config.key?(:index)

    return self.action_config[action] if self.action_config[action]
  end

  # No action_config, so try singular/plural config if explicitly instructed to via @many.
  return self.plural_config if @many == true && self.plural_config
  return self.singular_config if @many == false && self.singular_config

  # Lastly, try returning the default config, or singular/plural config in that order.
  return self.config || self.singular_config || self.plural_config
end

#get_serializer_configObject

Get a configuration passable to `serializable_hash` for the object, filtered if required.



218
219
220
# File 'lib/rest_framework/serializers.rb', line 218

def get_serializer_config
  return @_serializer_config ||= filter_except(self._get_raw_serializer_config)
end

#serialize(**kwargs) ⇒ Object



222
223
224
225
226
227
228
# File 'lib/rest_framework/serializers.rb', line 222

def serialize(**kwargs)
  if @object.respond_to?(:to_ary)
    return @object.map { |r| r.serializable_hash(self.get_serializer_config) }
  end

  return @object.serializable_hash(self.get_serializer_config)
end