Module: RESTFramework::BaseModelControllerMixin

Includes:
BaseControllerMixin
Included in:
ModelControllerMixin, ReadOnlyModelControllerMixin
Defined in:
lib/rest_framework/controller_mixins/models.rb

Overview

This module provides the core functionality for controllers based on models.

Defined Under Namespace

Modules: ClassMethods

Constant Summary collapse

RRF_BASE_MODEL_CONTROLLER_CONFIG =
{
  # Core attributes related to models.
  model: nil,
  recordset: nil,

  # Attributes for configuring record fields.
  fields: nil,
  field_config: nil,
  action_fields: nil,

  # Attributes for finding records.
  find_by_fields: nil,
  find_by_query_param: "find_by",

  # Attributes for create/update parameters.
  allowed_parameters: nil,
  allowed_action_parameters: nil,

  # Attributes for the default native serializer.
  native_serializer_config: nil,
  native_serializer_singular_config: nil,
  native_serializer_plural_config: nil,
  native_serializer_only_query_param: "only",
  native_serializer_except_query_param: "except",

  # Attributes for default model filtering, ordering, and searching.
  filterset_fields: nil,
  ordering_fields: nil,
  ordering_query_param: "ordering",
  ordering_no_reorder: false,
  search_fields: nil,
  search_query_param: "search",
  search_ilike: false,

  # Option for `recordset.create` vs `Model.create` behavior.
  create_from_recordset: true,

  # Control if filtering is done before find.
  filter_recordset_before_find: true,

  # Option to exclude associations from default fields.
  exclude_associations: false,

  # Control if bulk operations are done in a transaction and rolled back on error, or if all bulk
  # operations are attempted and errors simply returned in the response.
  bulk_transactional: false,

  # Control if bulk operations should be done in "batch" mode, using efficient queries, but also
  # skipping model validations/callbacks.
  bulk_batch_mode: false,
}

Constants included from BaseControllerMixin

RESTFramework::BaseControllerMixin::RRF_BASE_CONTROLLER_CONFIG

Class Method Summary collapse

Instance Method Summary collapse

Methods included from BaseControllerMixin

#api_response, #get_filtered_data, #options, #root, #rrf_error_handler, #serialize

Class Method Details

._rrf_bulk_transaction(&block) ⇒ Object

Create a transaction around the passed block, if configured. This is used primarily for bulk actions, but we include it here so it's always available.



471
472
473
474
475
476
477
# File 'lib/rest_framework/controller_mixins/models.rb', line 471

def self._rrf_bulk_transaction(&block)
  if self.bulk_transactional
    ActiveRecord::Base.transaction(&block)
  else
    yield
  end
end

.included(base) ⇒ Object



301
302
303
304
305
306
307
308
309
310
311
312
313
314
315
316
317
318
# File 'lib/rest_framework/controller_mixins/models.rb', line 301

def self.included(base)
  RESTFramework::BaseControllerMixin.included(base)

  return unless base.is_a?(Class)

  base.extend(ClassMethods)

  # Add class attributes (with defaults) unless they already exist.
  RRF_BASE_MODEL_CONTROLLER_CONFIG.each do |a, default|
    next if base.respond_to?(a)

    base.class_attribute(a)

    # Set default manually so we can still support Rails 4. Maybe later we can use the default
    # parameter on `class_attribute`.
    base.send(:"#{a}=", default)
  end
end

Instance Method Details

#_get_specific_action_config(action_config_key, generic_config_key) ⇒ Object



320
321
322
323
324
325
326
327
328
# File 'lib/rest_framework/controller_mixins/models.rb', line 320

def _get_specific_action_config(action_config_key, generic_config_key)
  action_config = self.class.send(action_config_key)&.with_indifferent_access || {}
  action = self.action_name&.to_sym

  # Index action should use :list serializer if :index is not provided.
  action = :list if action == :index && !action_config.key?(:index)

  return (action_config[action] if action) || self.class.send(generic_config_key)
end

#get_allowed_parametersObject

Get a list of parameters allowed for the current action. By default we do not fallback to columns so arbitrary fields can be submitted if no fields are defined.



365
366
367
368
369
370
# File 'lib/rest_framework/controller_mixins/models.rb', line 365

def get_allowed_parameters
  return _get_specific_action_config(
    :allowed_action_parameters,
    :allowed_parameters,
  ) || self.get_fields
end

#get_body_params(data: nil) ⇒ Object Also known as: get_create_params, get_update_params

Filter the request body for keys in current action's allowed_parameters/fields config.



385
386
387
388
389
390
391
392
393
394
395
396
397
398
399
400
401
402
403
404
405
406
407
408
409
410
411
412
413
414
415
# File 'lib/rest_framework/controller_mixins/models.rb', line 385

def get_body_params(data: nil)
  data ||= request.request_parameters

  # Filter the request body and map to strings. Return all params if we cannot resolve a list of
  # allowed parameters or fields.
  allowed_params = self.get_allowed_parameters&.map(&:to_s)
  body_params = if allowed_params
    data.select { |p| allowed_params.include?(p) }
  else
    data
  end

  # Add query params in place of missing body params, if configured.
  if self.class.accept_generic_params_as_body_params && allowed_params
    (allowed_params - body_params.keys).each do |k|
      if value = params[k].presence
        body_params[k] = value
      end
    end
  end

  # Filter primary key if configured.
  if self.class.filter_pk_from_request_body
    body_params.delete(self.class.get_model&.primary_key)
  end

  # Filter fields in exclude_body_fields.
  (self.class.exclude_body_fields || []).each { |f| body_params.delete(f) }

  return body_params
end

#get_fields(fallback: false) ⇒ Object

Get a list of fields for the current action. Returning `nil` indicates that anything should be accepted unless `fallback` is true, in which case we should fallback to this controller's model columns, or en empty array.



333
334
335
336
337
338
339
340
341
342
343
344
345
346
347
348
349
350
# File 'lib/rest_framework/controller_mixins/models.rb', line 333

def get_fields(fallback: false)
  fields = _get_specific_action_config(:action_fields, :fields)

  # If fields is a hash, then parse it.
  if fields.is_a?(Hash)
    return RESTFramework::Utils.parse_fields_hash(
      fields, self.class.get_model, exclude_associations: self.class.exclude_associations
    )
  elsif !fields && fallback
    # Otherwise, if fields is nil and fallback is true, then fallback to columns.
    model = self.class.get_model
    return model ? RESTFramework::Utils.fields_for(
      model, exclude_associations: self.class.exclude_associations
    ) : []
  end

  return fields
end

#get_filter_backendsObject

Get filtering backends, defaulting to using `ModelFilter` and `ModelOrderingFilter`.



378
379
380
381
382
# File 'lib/rest_framework/controller_mixins/models.rb', line 378

def get_filter_backends
  return self.class.filter_backends || [
    RESTFramework::ModelFilter, RESTFramework::ModelOrderingFilter
  ]
end

#get_find_by_fieldsObject

Get a list of find_by fields for the current action. Do not fallback to columns in case the user wants to find by virtual columns.



359
360
361
# File 'lib/rest_framework/controller_mixins/models.rb', line 359

def get_find_by_fields
  return self.class.find_by_fields || self.get_fields
end

#get_options_metadataObject

Pass fields to get dynamic metadata based on which fields are available.



353
354
355
# File 'lib/rest_framework/controller_mixins/models.rb', line 353

def 
  return self.class.
end

#get_recordObject

Get a single record by primary key or another column, if allowed. The return value is cached and exposed to the view as the `@record` instance variable.



442
443
444
445
446
447
448
449
450
451
452
453
454
455
456
457
458
459
460
461
462
463
464
465
466
467
# File 'lib/rest_framework/controller_mixins/models.rb', line 442

def get_record
  # Cache the result.
  return @record if instance_variable_defined?(:@record)

  recordset = self.get_recordset
  find_by_key = self.class.get_model.primary_key

  # Find by another column if it's permitted.
  if find_by_param = self.class.find_by_query_param.presence
    if find_by = params[find_by_param].presence
      find_by_fields = self.get_find_by_fields&.map(&:to_s)

      if !find_by_fields || find_by.in?(find_by_fields)
        find_by_key = find_by
      end
    end
  end

  # Filter recordset, if configured.
  if self.filter_recordset_before_find
    recordset = self.get_records
  end

  # Return the record. Route key is always `:id` by Rails convention.
  return @record = recordset.find_by!(find_by_key => request.path_parameters[:id])
end

#get_recordsObject

Get the records this controller has access to after any filtering is applied.



434
435
436
437
438
# File 'lib/rest_framework/controller_mixins/models.rb', line 434

def get_records
  return @records if instance_variable_defined?(:@records)

  return @records = self.get_filtered_data(self.get_recordset)
end

#get_recordsetObject

Get the set of records this controller has access to. The return value is cached and exposed to the view as the `@recordset` instance variable.



421
422
423
424
425
426
427
428
429
430
431
# File 'lib/rest_framework/controller_mixins/models.rb', line 421

def get_recordset
  return @recordset if instance_variable_defined?(:@recordset)
  return (@recordset = self.class.recordset) if self.class.recordset

  # If there is a model, return that model's default scope (all records by default).
  if (model = self.class.get_model(from_get_recordset: true))
    return @recordset = model.all
  end

  return @recordset = nil
end

#get_serializer_classObject

Get the configured serializer class, or `NativeSerializer` as a default.



373
374
375
# File 'lib/rest_framework/controller_mixins/models.rb', line 373

def get_serializer_class
  return super || RESTFramework::NativeSerializer
end